/**
 * Copyright (C) 2009-2011 enStratusNetworks LLC
 *
 * ====================================================================
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * ====================================================================
 */

package org.dasein.cloud.test;

import org.dasein.cloud.CloudException;
import org.dasein.cloud.CloudProvider;
import org.dasein.cloud.InternalException;
import org.dasein.cloud.platform.DatabaseEngine;
import org.junit.After;
import org.junit.Before;
import org.junit.Test;

public class RelationalDatabaseTestCase extends BaseTestCase {
    private CloudProvider              provider         = null;
    private Iterable<DatabaseEngine>   testEngines      = null;
    
    public RelationalDatabaseTestCase(String name) { super(name); }
    
    @Before
    @Override
    public void setUp() throws InstantiationException, IllegalAccessException, CloudException, InternalException {
        provider = getProvider();
        provider.connect(getTestContext());
        
        String name = getName();
        
        if( name.equals("testListVersions") || name.equals("testGetDefaultVersion") ) {
            testEngines = provider.getPlatformServices().getRelationalDatabaseSupport().getDatabaseEngines();
        }
    }
    
    @After
    @Override
    public void tearDown() {
        try {
            if( provider != null ) {
                provider.close();
            }
        }
        catch( Throwable t ) {
            t.printStackTrace();
        } 
    }
    
    @Test
    public void testGetDefaultVersion() throws InternalException, CloudException {
        begin();
        for( DatabaseEngine engine : testEngines ) {
            String version = provider.getPlatformServices().getRelationalDatabaseSupport().getDefaultVersion(engine);
            
            assertNotNull("Default version may not be null (" + engine + ")", version);
            out(engine.toString() + ": " + version);
        }
        end();
    }
    
    @Test
    public void testListEngines() throws InternalException, CloudException {
        begin();
        Iterable<DatabaseEngine> engines = provider.getPlatformServices().getRelationalDatabaseSupport().getDatabaseEngines();
        
        assertNotNull("Database engine listing may not be null", engines);
        assertTrue("Database engine list must contain at least one database engine", engines.iterator().hasNext());
        out("Supported database engines: ");
        for( DatabaseEngine engine : engines ) {
            out("\t-> " + engine.toString());
        }
        end();
    }
    
    @Test
    public void testListVersions() throws InternalException, CloudException {
        begin();
        for( DatabaseEngine engine : testEngines ) {
            Iterable<String> versions = provider.getPlatformServices().getRelationalDatabaseSupport().getSupportedVersions(engine);
            
            assertNotNull("Versions may not be null (" + engine + ")", versions);
            assertTrue("Database engine (" + engine + ") must have at least one supported version.", versions.iterator().hasNext());
            out(engine.toString());
            for( String version : versions ) {
                out("\t-> " + version);
            }
        }
        end();
    }
}
